// *****************************************************************************
//
// Copyright. 2007. Symvionics, Inc. All rights reserved.
// 
// Symvionics Inc. ("Symvionics") grants a non-exclusive license for you to use
// this source code and the accompanying documentation, if any, free of charge,
// provided that this paragraph, and the following two paragraphs appear in all
// copies and in all derivative works.  In no event shall this source code or
// this license thereto impact upon or fall within the scope of any other
// agreement that you have with Symvionics.
// 
// BY USING THIS SOFTWARE, YOU FURTHER AGREE THAT, IN NO EVENT, SHALL SYMVIONICS
// BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
// SOFTWARE, THE ACCOMPANYING DOCUMENTATION, IF ANY, OR ANY DERIVATIVE WORKS.
// YOU AGREE TO DEFEND, INDEMNIFY, AND HOLD HARMLESS SYMVIONICS FROM AND AGAINST
// ALL FINES, PENALTIES, LIABILITIES, DAMAGES, COSTS, AND EXPENSES INCURRED
// BY SYMVIONICS AS A RESULT OF YOUR USE OF THIS SOFTWARE INCLUDING THIRD PARTY
// USE OF ANY DERIVATIVE WORKS CREATED BY YOU.
// 
// SYMVIONICS SPECIFICALLY DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE.  THE SOFTWARE AND ACCOMPANYING DOCUMENTATION LICENSED
// HEREUNDER ARE PROVIDED "AS IS" WITHOUT WARRANTY OR OBLIGATION OF SUPPORT OR 
// MAINTENANCE.
//
// *****************************************************************************
//
// 1) Overview
//
//   This program demonstrates using the IadsDataFileInterface COM API.  This interface supports the
// new IADS file structure for version 5.0.  This version represents a significant upgrade from previous
// IADS files, including; new file extensions, a new index file type, expanded data formats, and 2 new data types.
// ** Plesae note that the existing "LittleEndianCDSFile" format will continue to be supported.
//
// 2) What's new in 5.0:
//
//   New files and extensions
//      - New extension for data - "*.IadsData"
//      - New Index File - "*.IadsIndex" - This is used for both Aperioidic and Multi-Periodic data types
// 
//   Additional formats:
//      - Int - Pre 5.0
//      - float 32 - Pre 5.0
//      - unsigned Int - Pre 5.0    
//      - Float64 (Double) - *New*
//      - Int64 - *New*
//      - Unsigned Int64 - *New*
//      - Binary Objects - any static size (Blobs) - *New*
//      - 1 byte Ascii strings - *New* - (Not Yet Supported)
//
//   New DataTypes 
//      - Peridoic - pre 5.0
//      - Aperiiodic - *New*
//      - Multi-Periodic - *New* 
//
// 3) Notes on using the IIadsDataFileInterface COM API
//     - The read routines operate in two modes, read with a start time or "sequential" mode useful for
//       consecutive reads, works well for reading through the entire file.
//     - A time data array passed to Append call for a Periodic data file is ignored
//     - the Data size for files using scalars type will automatically set the data size to either 4 or 8
//     - It's important to set the data type, otherwise an error will be returned.  If IadsAsyncType is set
//       and the sample rate is not set it will be set to 10.0 - this becomes the suggested iteration sample rate.
//       If the data type is set to IadsPeriodicType and the sample rate is <= 0.0 then an error is retuned.
//     - When reading data the DataFormat type must be set
//     - All File reads and writes leave the file pointer in the last position - in this way
//       contigueous multiple reads/writes can be accomplished.
//     - When creating new files only the Sample rate, Start time and Data format are required, unless creating
//       Blob types, then the data size property must be set.  The End time might not be set in the file by this API
//       this is not an error in IADS and can be safely ignored.
//     - Time is entered as a 64 bit integer count in nanoseconds from Jan 1
//
// 4) What does the Data Tester test program demonstrate ?
//      - This program was not written with performance in mind - only for demonstration purposes
//      - You can NOT use the "Get Next" style of reading (StartTime = -1.0), with multi-periodic files
//      - This program was written to test as much of the API as possible, it creates all formats of the
//        Periodic type, several for the APeriodic and Multi-periodic formats. It also reads the data
//        and displays it within the console window.
//
//
// 5) Support Programs - the "IadsDataViewer.exe"
//      - This program will display any type of data in a tabular format for easy debugging purposes
//      - Make sure and register the IadsGrid.dll in order for it to work
//      - The top window displays the header, certain values like the sample rate and start time can
//        can be modified and written back to the file
//      - you can enter in a time strinmg to jump to in either an Irig time string, total seconds, or total
//        nanoseconds
//      - Visible Data can also be exported to a CSV file from this viewer using the File/export option
//      - For Multi-periodic data type, you can move by segment using the "segment" check box
//        
//
#include <atlbase.h>

#include "IadsDataFileInterface.h"
#include "IadsDataFileInterface_i.c"

#include "IadsTime.h"
#include "IadsTime_i.c"


// Set these to isolate specific test areas
//#define CREATE_PERIODIC_DATA
//#define READ_PERIODIC_DATA
//#define READ_PERIODIC_DATA2
//#define READ_DATA
//#define APPEND_DATA
//#define AUTOMATION_PERF_TEST

//#define READ_GET_NEXT_STYLE
//#define READ_CLASSIC_DATA

//#define CREATE_APERIODIC_DATA
//#define READ_APERIODIC_DATA

#define CREATE_MULTI_PERIODIC_DATA
#define READ_MULTI_PERIODIC_DATA

//#define SCRIPT_METHODS_TEST
//#define START_TIME_TEST

const __int64 cNanoSec = 1000000000 ;

#pragma warning( disable : 4101 ) // unreferenced variables


IDataFile* IadsData = NULL ;


template <class floatType>
class tPerformanceTimer
{

public:
  tPerformanceTimer( void )
  {
    LARGE_INTEGER countsPerSecond;
    QueryPerformanceFrequency( &countsPerSecond );
    if( countsPerSecond.QuadPart == (__int64)0 )
    {
      mCountsPerSecond = 1;
      mCountsPerMillisecond = 1000;
    }
    else
    {
      mCountsPerSecond = countsPerSecond.QuadPart;
      mCountsPerMillisecond = mCountsPerSecond / (__int64)1000;
    }
    mStartTime  = (__int64)0;
    mStopTime   = (__int64)0;
  }

  ~tPerformanceTimer( void )
  {
  }

  HRESULT start( void )
  {
    LARGE_INTEGER startTime;
    QueryPerformanceCounter( &startTime );
    mStartTime = startTime.QuadPart;
    return S_OK;
  }

  HRESULT stop( void )
  {
    LARGE_INTEGER stopTime;
    QueryPerformanceCounter( &stopTime );
    mStopTime = stopTime.QuadPart;
    return S_OK;
  }

  floatType diff( void )
  {
    __int64 diffInCounts = mStopTime - mStartTime;
    floatType elapsedTimeInSeconds = (floatType)diffInCounts / (floatType)mCountsPerSecond;
    return elapsedTimeInSeconds;
  }

  floatType diffMilliseconds( void )
  {
    __int64 diffInCounts = mStopTime - mStartTime;
    floatType elapsedTimeInMilliseconds = (floatType)diffInCounts / (floatType)mCountsPerMillisecond;
    return elapsedTimeInMilliseconds;
  }
  
  __int64 mCountsPerSecond;
  __int64 mCountsPerMillisecond;
  __int64 mStartTime;
  __int64 mStopTime;

};
  

void shutdown(void)
{
  if( IadsData != NULL ) IadsData->Release();

  ::CoUninitialize();

	printf("\n\nFinished...Hit any key to continue\n");
  getchar();
	exit( 0 );
}


//
// Main Program
//
void main(int argc, char* argv[])
{
  // Initialize the Microsoft COM system
  ::CoInitialize(NULL);

  // Create the IADS config object
  HRESULT hr = ::CoCreateInstance( CLSID_DataFile, NULL, CLSCTX_ALL, IID_IDataFile, (void**)&IadsData );
  if( hr != S_OK || IadsData == NULL )
  {
    printf("** Unable to create the IadsDataFileInteface object ?\n" );
    shutdown();
  }  

  IIadsTime* timebaby = NULL ;
  CoCreateInstance( CLSID_IadsTime, NULL, CLSCTX_INPROC_SERVER, IID_IIadsTime, (void**)&timebaby) ;

/******************************************************************************
 *
 *                         Creating IADS Files
 *
 ******************************************************************************/

  BSTR bFileName[3] ;
  bFileName[0] = ::SysAllocString( L"Parm1" ) ;
  bFileName[1] = ::SysAllocString( L"Parm2" ) ;
  bFileName[2] = ::SysAllocString( L"Parm3" ) ;

//
// Make Data - Fill our example data types with a repeating ramp and nice even times for debugging
//
  float fData[5000] ;
  double dData[5000] ;
  int iData[5000] ;
  unsigned int uiData[5000] ;
  __int64 i64Data[5000] ;
  unsigned __int64 ui64Data[5000] ;
  __int64 timeData[5000] ;
  __int64 NewTimeData[5000] ;
  unsigned char bData[5] = {0x04, 0x03, 0x02, 0x01, 0x66 } ;
  char aData[12] = {"Hello World"} ;
  long anp ;
  int df ;
  double minTimeInSec ;
  double slope ;
 
  int val = 0 ;
  for( register int v=0; v < 5000; v++ )
  {
    fData[v]    = (float)val ;
    dData[v]    = (double)val ;
    iData[v]    = (int)val ;
    uiData[v]   = (unsigned int)val ;
    i64Data[v]  = val < 250?(__int64)(val*-1):(__int64)val ;
    ui64Data[v] = (unsigned __int64)val ;

    if( ++val > 499 ) val = 0 ;
  }

//
//  Periodic Data
//
#ifdef CREATE_PERIODIC_DATA

  printf("*) Create Periodic Sample Files\n") ;

  //Make sure these diretoies exist
  ::CreateDirectory( ".\\Data", NULL ) ;
  ::CreateDirectory( ".\\Data\\Periodic", NULL ) ;
  ::CreateDirectory( ".\\Data\\Periodic\\Float", NULL ) ;
  ::CreateDirectory( ".\\Data\\Periodic\\Double", NULL  ) ;
  ::CreateDirectory( ".\\Data\\Periodic\\Int", NULL ) ;
  ::CreateDirectory( ".\\Data\\Periodic\\Int64", NULL )  ;
  ::CreateDirectory( ".\\Data\\Periodic\\Blob", NULL ) ;
  ::CreateDirectory( ".\\Data\\Periodic\\UInt", NULL ) ;
  ::CreateDirectory( ".\\Data\\Periodic\\UInt64", NULL ) ;

  // Note: variable df doesn't match the enum type (in case you were wondering) it's just a pass counter
  for( df=0; df < 7; df++ )
  {
    if( df == 0 )
      printf("\t- Creating Float data\n") ;
    else if (df == 1 )
      printf("\t- Creating Double data\n") ;
    else if (df == 2 )
      printf("\t- Creating Int data\n") ;
    else if (df == 3 )
      printf("\t- Creating Int64 data\n") ;
   else if (df == 4 )
      printf("\t- Creating Blob data\n") ;
   else if (df == 5 )
      printf("\t- Creating Unsigned Int data\n") ;
   else if (df == 6 )
      printf("\t- Creating Unsigned Int 64 data\n") ;

    // Number of Parms per data format type
    for( register int n=0; n < 3; n++ )
    {

      if( df == 0 )
        hr = IadsData->Create( L".\\Data\\Periodic\\Float",  bFileName[n] ) ;
      else if (df == 1 )
        hr = IadsData->Create( L".\\Data\\Periodic\\Double", bFileName[n] ) ;
      else if( df == 2 )
        hr = IadsData->Create( L".\\Data\\Periodic\\Int",    bFileName[n] ) ;
      else if( df == 3 )
        hr = IadsData->Create( L".\\Data\\Periodic\\Int64",  bFileName[n] ) ;
      else if( df == 4 )
        hr = IadsData->Create( L".\\Data\\Periodic\\Blob",   bFileName[n] ) ;
      else if( df == 5 )
        hr = IadsData->Create( L".\\Data\\Periodic\\UInt",   bFileName[n] ) ;
      else if( df == 6 )
        hr = IadsData->Create( L".\\Data\\Periodic\\UInt64", bFileName[n] ) ;

      if( hr != S_OK ) continue;

      // Write the needed properties for to our file, here's what is required:
      //   1) SampleRate  - a double - if you have a non-integer sample rate use has much precision as possible, we accept to nanoseocnd resolution 
      //   2) Start Time  - Double represeting Irig time in Seconds.Subseocnds
      //   3) Data Format - Need to tell us what type of data is going to be written - see the IadsDataFileInterface IDL file
      //   4) Data Size   - Needed if writing Blobs - otherwise computed internally for all scalar types

      IadsData->put_StartTime( 86400*cNanoSec ); // we want all files to start at the same time
      IadsData->put_SampleRate( 50.123456789 ) ;
      IadsData->put_DataType( IadsPeriodicType ) ;

      if( df == 0 )
      {
        IadsData->put_DataFormat( IadsFloat32Type ) ;
      }
      else if( df == 1 )
      {
        IadsData->put_DataFormat( IadsFloat64Type ) ;
      }
      else if( df == 2 )
      {
        IadsData->put_DataFormat( IadsInt32Type ) ;
      }
      else if( df == 3 )
      {
        IadsData->put_DataFormat( IadsInt64Type ) ;
      }
      else if( df == 4 )
      {
        IadsData->put_SampleRate( 50.0 ) ;
        IadsData->put_DataFormat( IadsBlobType ) ;
        IadsData->put_DataSize(5) ;
      }
      else if( df == 5 )
      {
        IadsData->put_DataFormat( IadsUnsignedInt32Type ) ;
      }
      else if( df == 6 )
      {
        IadsData->put_DataFormat( IadsUnsignedInt64Type ) ;
      }
 
      // Important to call WriteHeader once all the properties have been set
      hr = IadsData->WriteHeader();
      if( hr != S_OK ) continue ;

      // Repeat the ramp a couple of times to show multiple append calls
      if( df == 0 )
      {
        IadsData->AppendDataF32( 5000, fData, &anp, NULL ) ;
        IadsData->AppendDataF32( 5000, fData, &anp, NULL ) ;
      }
      else if( df == 1 )
      {
        IadsData->AppendDataF64( 5000, dData, &anp, NULL ) ;
        IadsData->AppendDataF64( 5000, dData, &anp, NULL ) ;
      }
      else if( df == 2 )
      {
        IadsData->AppendDataI32( 5000, iData, &anp, NULL ) ;
        IadsData->AppendDataI32( 5000, iData, &anp, NULL ) ;
      }
      else if( df == 3 )
      {
        IadsData->AppendDataI64( 5000, i64Data, &anp, NULL ) ;
        IadsData->AppendDataI64( 5000, i64Data, &anp, NULL ) ;
      }
      else if( df == 4 )
      {
        for( register int p=0; p < 2000; p++ )
          IadsData->AppendDataBlob( 1, bData, &anp, NULL ) ;
      }
      else if( df == 5 )
      {
        IadsData->AppendDataUI32( 5000, uiData, &anp, NULL ) ;
        IadsData->AppendDataUI32( 5000, uiData, &anp, NULL ) ;
      }
      else if( df == 6 )
      {
        IadsData->AppendDataUI64( 5000, ui64Data, &anp, NULL ) ;
        IadsData->AppendDataUI64( 5000, ui64Data, &anp, NULL ) ;
      }
 
      IadsData->Close() ;
    }
  }

#endif

//
// Craete APeriodic Data
//
#ifdef CREATE_APERIODIC_DATA

  //Make sure these diretoies exist
  ::CreateDirectory( ".\\Data", NULL ) ;
  ::CreateDirectory( ".\\Data\\APeriodic", NULL ) ;
  ::CreateDirectory( ".\\Data\\APeriodic\\Float", NULL ) ;
  ::CreateDirectory( ".\\Data\\APeriodic\\Double", NULL ) ;

  printf("\n*) Create APeriodic Sample Files - Float and Double type\n") ;

  timeData[0] = 86400*cNanoSec ;
  minTimeInSec = 1.0/200.0 ;
  slope = 200.0/(float)RAND_MAX ;
  for(int tt=1; tt < 5000; tt++ )
  {
    double y = slope * (float)::rand() ;
    timeData[tt] = timeData[tt-1] + (__int64)((y*minTimeInSec)*1E9) ; 
  }

  // Make a Time Array  with random time values to simulate an Aperiodic parameter
  if( IadsData->Create( L".\\Data\\APeriodic\\Float",  bFileName[0] ) == S_OK )
  {
    IadsData->put_StartTime( 86400*cNanoSec ); // we want all files to start at the same time
    // Although the samlpe rate is not required - this allows the user to set the 
    // suggested iteration sample rate for displays that will continue to fill data, like strip charts.
    IadsData->put_SampleRate( 50.0 ) ; 
    IadsData->put_DataFormat( IadsFloat32Type ) ;
    IadsData->put_DataType( IadsAsyncType ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      // This automatically updates the data file and index file
      IadsData->AppendDataF32( 5000, fData, &anp, timeData ) ;
    }
    IadsData->Close() ;
  }

  if( IadsData->Create( L".\\Data\\APeriodic\\Double",  bFileName[0] ) == S_OK )
  {
    IadsData->put_StartTime( 86400*cNanoSec ); // we want all files to start at the same time
    IadsData->put_SampleRate( 30.0 ) ; // Critical - this is waht denotes an Aperiodic data type
    IadsData->put_DataType( IadsAsyncType ) ;
    IadsData->put_DataFormat( IadsFloat64Type ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      // This automatically updates the data file and index file
      IadsData->AppendDataF64( 5000, dData, &anp, timeData ) ;
    }
    IadsData->Close() ;
  }

#endif

// 
// Create Multi-Periodic Data
// 
#ifdef CREATE_MULTI_PERIODIC_DATA

  ::CreateDirectory( ".\\Data", NULL ) ;
  ::CreateDirectory( ".\\Data\\MultiPeriodic", NULL ) ;
  ::CreateDirectory( ".\\Data\\MultiPeriodic\\Float", NULL ) ;
  ::CreateDirectory( ".\\Data\\MultiPeriodic\\Double", NULL ) ;

  printf("\n*) Create Multi-Periodic Sample Files\n") ;

  if( IadsData->Create( L".\\Data\\MultiPeriodic\\Float",  bFileName[0] ) == S_OK )
  {
    IadsData->put_StartTime( 86400*cNanoSec ); // we want all files to start at the same time
    IadsData->put_SampleRate( 100.0 ) ;
    IadsData->put_DataType( IadsPeriodicType ) ;
    IadsData->put_DataFormat( IadsFloat32Type ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      // This automatically updates the data file and index file
      IadsData->AppendDataF32( 5000, fData, &anp, NULL ) ;
      // This is the start of the new segement - this leaves a hole between the data for our test purposes
      IadsData->AppendSegment( 87000*cNanoSec ) ; 
      IadsData->AppendDataF32( 5000, fData, &anp, NULL ) ;
      IadsData->AppendSegment( 90000*cNanoSec ) ; 
      IadsData->AppendDataF32( 5000, fData, &anp, NULL ) ;
      IadsData->AppendSegment( 91000*cNanoSec ) ; 
      IadsData->AppendDataF32( 5000, fData, &anp, NULL ) ;
    }
    // *Manditory - One final Append, time must be set to 0.0 
    // MRB: No final append call needed...in fact downstream will assume this is start of new segment...
//    IadsData->AppendSegment( 0 ) ;
    IadsData->Close() ;
  }

  if( IadsData->Create( L".\\Data\\MultiPeriodic\\Double",  bFileName[0] ) == S_OK )
  {
    IadsData->put_StartTime( 86400*cNanoSec ); // we want all files to start at the same time
    IadsData->put_SampleRate( 100.0 ) ; // Critical - this is waht denotes an Aperiodic data type
    IadsData->put_DataType( IadsPeriodicType ) ;
    IadsData->put_DataFormat( IadsFloat64Type ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      // This automatically updates the data file and index file
      IadsData->AppendDataF64( 5000, dData, &anp, NULL ) ;
      // This is the start of the new segement - this leaves a hole between the data
      IadsData->AppendSegment( 86500*cNanoSec ) ; 
      IadsData->AppendDataF64( 5000, dData, &anp, NULL ) ;
      IadsData->AppendSegment( 86600*cNanoSec ) ; 
      IadsData->AppendDataF64( 5000, dData, &anp, NULL ) ;
    }
    // One final Append 
    // MRB: No final append call needed...in fact downstream will assume this is start of new segment...
//    IadsData->AppendSegment( 0 ) ;
    IadsData->Close() ;
  }


#endif


/******************************************************************************
 *
 *                         Reading IADS Data Files that we just wrote
 *
 ******************************************************************************/

  // Read and dump the header
  double sampleRate ;
  __int64 startTime, endTime ;
  IadsDataFormat dataFormat ;
  IadsDataType dataType ;
  int fileSize, totalPoints ;


//
// Check a start Time bug and test the IIadsTime CoCreate
//
#ifdef AUTOMATION_PERF_TEST

  hr = IadsData->OpenExisting( L"d:\\PostFlightData\\IADSTestData\\NewIadsData\\FixedDataSize\\PeriodicFloat",  L"AB001.IadsData" ) ;
  //hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Int64",  L"parm1" ) ;
  //hr = IadsData->OpenExisting( L".\\Data\\Periodic\\UInt64",  L"parm1" ) ;
  //hr = IadsData->OpenExisting( L".\\Data\\APeriodic\\Double\\",  L"parm1" ) ;
  //hr = IadsData->OpenExisting( L"d:\\PostFlightData\\IADSTestData\\NewIadsData\\FixedDataSize\\PeriodicBlob2",  L"BlobParm" ) ;

  if( hr != S_OK ) return ;

  IIadsTime* sTime ;
  IIadsTime* eTime ;

  IadsData->get_SampleRate( &sampleRate );
  IadsData->get_StartTimeVB( &sTime );
  IadsData->get_EndTimeVB( &eTime);
  IadsData->get_DataFormat( &dataFormat) ;
  IadsData->get_DataType( &dataType ) ;
  IadsData->get_FileSize( &fileSize ) ;
  IadsData->get_TotalPoints( &totalPoints ) ;


  __int64 sNano, eNano ;
  sTime->get_TotalNanoseconds(&sNano) ;
  eTime->get_TotalNanoseconds(&eNano) ;

  printf( "Header Information:\n\n" );
  printf( "  Sample Rate: %f\n", sampleRate );
  printf( "    StartTime: %I64d\n", sNano );
  printf( "      EndTime: %I64d\n", eNano );
  printf( "  Data Format: %d\n", (int)dataFormat );
  printf( "    Data type: %d\n", (int)dataType );
  printf( "    File Size: %d\n", fileSize );
  printf( "Total Samples: %d\n", totalPoints );

  __int64 st1_64, st2_64 ;
  IadsData->get_StartTime(&st1_64) ;

  VARIANT irigTime ; irigTime.vt = VT_EMPTY ;
  VARIANT data ; data.vt = VT_EMPTY ;
  VARIANT highData; highData.vt = VT_EMPTY ;
  VARIANT lowData; lowData.vt = VT_EMPTY ;
  VARIANT vanp ;

  if( dataFormat == IadsUnsignedInt64Type || dataFormat == IadsInt64Type )
  {
    IadsData->ReadData64( sTime, totalPoints, &highData, &lowData, &vanp, &irigTime ) ;
    unsigned int* hn = (unsigned int*)highData.parray->pvData ;
    unsigned int* ln = (unsigned int*)lowData.parray->pvData ;
    if( hn && ln )
    {
      __int64 data64 ;
      for(int p=0; p < 20; p++ )
      {
        data64 = hn[p] ;
        data64 = (data64<<=32)|ln[p] ;
        printf("Data Value = %I64d\n", data64) ;
      }
    }
  }
  else
  {
    IadsData->ReadData( sTime, 100, &data, &vanp, &irigTime ) ;

    IadsDataFormat df ;
    IadsData->get_DataFormat(&df) ;
    if( df == IadsBlobType )
    {
      unsigned char* c = (unsigned char*)data.parray->pvData ;
      printf("%02x-%02x-%02x-%02x\n", c[0], c[1], c[2], c[3] ) ;
      printf("%f, %f, %f, %f\n", *(float*)&c[0], *(float*)&c[4], *(float*)&c[8], *(float*)&c[12] ) ;
    }
  }
  IadsData->Close() ;

  // Make a Time Array  with random time values to simulate an Aperiodic parameter
  if( IadsData->Create( L"D:\\",  L"param2" ) == S_OK )
  {
    IadsData->put_SampleRate(sampleRate) ;
    IadsData->put_DataFormat(dataFormat) ;
    IadsData->put_DataType(dataType) ;

    sTime->put_TotalNanoseconds(st1_64) ;
    IadsData->put_StartTimeVB( sTime ) ; // the start time from the opened file
    IadsData->get_StartTime(&st2_64) ;

    printf("\nparm1 start time: %I64d\nparam1 start time: %I64d\n", st1_64, st2_64 ) ; 

     ;
    IadsData->WriteHeader() ;
    if( dataFormat == IadsUnsignedInt64Type || dataFormat == IadsInt64Type )
        IadsData->AppendData64( vanp.intVal, &highData, &lowData, &vanp, &irigTime ) ;
    else
        IadsData->AppendData( vanp.intVal, &data, &vanp, &irigTime ) ;
    IadsData->Close() ;
  }

  tPerformanceTimer<double> t ; t.start() ;

#ifdef METHOD1
  // Test the CoCreate
  IIadsTime* time = NULL ;
  for(int n=0; n < 10000; n++ )
  {
    HRESULT hr = CoCreateInstance( CLSID_IadsTime, NULL, CLSCTX_INPROC_SERVER, IID_IIadsTime, (void**)&time) ;
    if( hr != S_OK )
      int i = 10 ;

    time->put_NanosecondsPart(500000000) ;
  }
#else

  ::VariantClear(&irigTime) ;
  IadsData->CreateIadsTimeArray( 10000, &irigTime ) ;

  IDispatch** pt = (IDispatch**)irigTime.parray->pvData ;

  IIadsTime* time ;
               
  __int64 tn = 86400000000000 ;
  for( int n=0; n < 10000; n++ )
  {
    if( pt && pt[n] )
    {
      pt[n]->QueryInterface(IID_IIadsTime, (void**)&time ) ;
      time->put_TotalNanoseconds(tn + (n*10000)) ;
      time->Release() ;
    }
  }
#endif

  t.stop() ;
  printf("\nTime to create 10000 IIadsTime objects(ms): %f\n", t.diffMilliseconds() ) ;

  t.start() ;
 // Test the routines to help with Time object performance
  VARIANT highInt ; highInt.vt = VT_EMPTY ;
  VARIANT lowInt ; lowInt.vt = VT_EMPTY ;
  hr = IadsData->FromIadsTimeToInt( &irigTime, &highInt, &lowInt ) ;

  t.stop() ;
  printf("\nTime to call FromIadsTimeToInt with 10000 IIadsTime objects(ms): %f\n", t.diffMilliseconds() ) ;

  unsigned int* hn = (unsigned int*)highInt.parray->pvData ;
  unsigned int* ln = (unsigned int*)lowInt.parray->pvData ;

  __int64 newNano ;
  for(int p=0; p < 100; p++ )
  {
    newNano = hn[p] ;
    newNano = (newNano<<=32)|ln[p] ;
    printf("Nano Seconds from Ints Array = %I64d\n", newNano) ;
  }

  ::VariantClear(&irigTime) ;

  t.start() ;
  IadsData->FromIntToIadsTime( &highInt, &lowInt, &irigTime ) ;
  t.stop() ;
  printf("\nTime to call FromIntToIadsTime with 10000 IIadsTime objects(ms): %f\n", t.diffMilliseconds() ) ;

  IUnknown** uTime = (IUnknown**)irigTime.parray->pvData ;
  time= NULL ;
  for( int n=0; n < 100; n++ )
  {
    if( uTime && uTime[n] )
    {
      uTime[n]->QueryInterface(IID_IIadsTime, (void**)&time) ;
      if( time )
      {
        __int64 nano ;
        time->get_TotalNanoseconds(&nano) ;
        printf("Nano Seconds from Iads Time Array = %I64d\n", nano) ;
        time->Release() ; time = NULL ;
      }
    }
  }
 
#endif

//
// Read the various periodic data types
//
#ifdef READ_DATA

  printf("\n*) Read and Display Periodic Sample Files Using VB Based Routines\n") ;

  IIadsTime* time = NULL ;
  CoCreateInstance( CLSID_IadsTime, NULL, CLSCTX_INPROC_SERVER, IID_IIadsTime, (void**)&time) ;

  for( df=0; df < 6; df++ )
  {
    // Open existing IADS data file. Make sure the file is not "read only", because this funtion will fail.
    // It's written for general updating/creating of file only..
    if( df == 0 ) 
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Float",  bFileName[0] ) ;
    else if( df == 1 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Double", bFileName[0] ) ;
    else if( df == 2 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Int",    bFileName[0] ) ;
    else if( df == 3 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\UInt",   bFileName[0] ) ;
    else if( df == 4 )
      hr = IadsData->OpenExisting( L".\\Data\\Aperiodic\\Float",   bFileName[0] ) ;
    else if( df == 5 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Int",   bFileName[0] ) ;

    if( hr != S_OK ) continue;

    IadsData->get_SampleRate( &sampleRate );
    IadsData->get_StartTime( &startTime );
    IadsData->get_EndTime( &endTime);
    IadsData->get_DataFormat( &dataFormat) ;
    IadsData->get_DataType( &dataType ) ;
    IadsData->get_FileSize( &fileSize ) ;
    IadsData->get_TotalPoints( &totalPoints ) ;

    printf( "Header Information:\n\n" );
    printf( "  Sample Rate: %f\n", sampleRate );
    printf( "    StartTime: %I64d\n", startTime );
    printf( "      EndTime: %I64d\n", endTime );
    printf( "  Data Format: %d\n", (int)dataFormat );
    printf( "    Data type: %d\n", (int)dataType );
    printf( "    File Size: %d\n", fileSize );
    printf( "Total Samples: %d\n", totalPoints );

    long actualPoints ;

    // Read and print out the first 50 data points from the file
    // 1) StartTime: -1.0 for a startime means to read fromthe start of the file leaving the pointer for the next read
    // 2) RequestedPonts - 50 total number of Points to read
    // 3) Data - the data Array
    // 4) Actual Points - The number of points that were read and returned in the data array
    // 5) TimeArray - Optional - filled in if not null - works for both periodic and Aperiodic

    unsigned char b50Data[250] ;

    if( time ) time->put_TotalNanoseconds(startTime) ;
    //if( time ) time->put_TotalNanoseconds(-1) ; // Test GetNextStyle

    VARIANT irigTime ; irigTime.vt = VT_EMPTY ;
    VARIANT data ; data.vt = VT_EMPTY ;

    if( df < 5 )
      HRESULT hr = IadsData->ReadData( time, 600, &data, &actualPoints, &irigTime ) ;
    else
      HRESULT hr = IadsData->ReadData( time, 600, &data, &actualPoints, NULL ) ;

    int p ;
    if( hr == S_OK )
    {
      IDispatch** pt = NULL ;
      IIadsTime* time = NULL ;
      __int64 t = -1 ;

      if( df < 5 )
      {
        pt = (IDispatch**)irigTime.parray->pvData ;
      }

      for( p=0; p < actualPoints; p++ )
      {
        if( df < 5 )
        {
          if( pt && pt[p] )
          {
            pt[p]->QueryInterface(IID_IIadsTime, (void**)&time ) ;
            time->get_TotalNanoseconds(&t) ;
            time->Release() ;
          }
        }
        if( data.vt == (VT_ARRAY|VT_R4) && dataType != IadsAsyncType  ) 
        {
          float* d = (float*)data.parray->pvData ;
          printf( "%I64d, Data[%d], %f\n", t, p, d[p]  );
        }
        else if( data.vt == (VT_ARRAY|VT_R4) && dataType == IadsAsyncType  ) 
        {
          float* d = (float*)data.parray->pvData ;
          printf( "%I64d, Data[%d], %f\n", t, p, d[p]  );
        }
        else if( data.vt == (VT_ARRAY|VT_R8) )
        {
          double* d = (double*)data.parray->pvData ;
          printf( "%I64d, Data[%d], %lf\n", t, p, d[p]  );
        }
        else if( data.vt == (VT_ARRAY|VT_I4) && df != 5 )
        {
          int* d = (int*)data.parray->pvData ;
          printf( "%I64d, Data[%d], %d\n", t, p, d[p]  );
        }
        else if( data.vt == (VT_ARRAY|VT_I4) && df == 5 )
        {
          int* d = (int*)data.parray->pvData ;
          printf(" Data[%d], %d\n", p, d[p]  );
        }
        else if( data.vt == (VT_ARRAY|VT_UI4) )
        {
          unsigned int* d = (unsigned int*)data.parray->pvData ;
          printf( "%I64d, Data[%d], %ud\n", t, p, d[p]  );
        }
      }
      VariantClear(&irigTime) ;
      VariantClear(&data) ;

      printf("Hit any key for the next data type\n") ;
      getchar() ;
    }
    IadsData->Close();
  }

#endif

#ifdef APPEND_DATA

  printf("\n*) Create APeriodic Sample Files,Float and Double using the AppendData() routine\n") ;

  timeData[0] = 86400*cNanoSec ;
  minTimeInSec = 1.0/200.0 ;
  slope = 200.0/(float)RAND_MAX ;
  for(int tt=1; tt < 5000; tt++ )
  {
    double y = slope * (float)::rand() ;
    timeData[tt] = timeData[tt-1] + (__int64)((y*minTimeInSec)*1E9) ; 
  }

  IIadsTime* st = NULL ;
  CoCreateInstance( CLSID_IadsTime, NULL, CLSCTX_INPROC_SERVER, IID_IIadsTime, (void**)&st) ;

  // Make a Time Array  with random time values to simulate an Aperiodic parameter
  if( IadsData->Create( L".\\Data\\APeriodic\\Float",  L"Float2" ) == S_OK )
  {
    if( st )
    {
      st->put_TotalNanoseconds( 86400*cNanoSec ) ;
      IadsData->put_StartTimeVB( st );
      st->Release() ;
    }

    // Although the samlpe rate is not required - this allows the user to set the 
    // suggested iteration sample rate for displays that will continue to fill data, like strip charts.
    IadsData->put_SampleRate( 50.0 ) ; 
    IadsData->put_DataFormat( IadsFloat32Type ) ;
    IadsData->put_DataType( IadsAsyncType ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      // This automatically updates the data file and index file
      // Data array
      VARIANT dv; dv.vt = (VT_ARRAY|VT_R4) ;
      SAFEARRAYBOUND rgsabound[1]; rgsabound[0].lLbound = 0;  rgsabound[0].cElements = 5000;
      dv.parray = ::SafeArrayCreate( VT_R4, 1, rgsabound) ;
      dv.parray->pvData = (float*)fData ;

      VARIANT tv; tv.vt = (VT_ARRAY|VT_DISPATCH) ;
      rgsabound[0].lLbound = 0;  rgsabound[0].cElements = 5000;
      tv.parray = ::SafeArrayCreate( VT_DISPATCH, 1, rgsabound) ;
      IDispatch** pDisp = (IDispatch**)tv.parray->pvData ;
      for( int n=0; n < 5000; n++)
      {
        IIadsTime* time = NULL ; IDispatch* dTime ;
        CoCreateInstance( CLSID_IadsTime, NULL, CLSCTX_INPROC_SERVER, IID_IIadsTime, (void**)&time) ;
        time->put_TotalNanoseconds( 86400000000000+(n*10000) ) ;
        pDisp[n] = static_cast<IDispatch*>(time) ;
      }
      long anp ;
      IadsData->AppendData( 5000, &dv, &anp, &tv ) ;
    }
    IadsData->Close() ;
  }

//
// Now open the file and show the data
// 
  hr = IadsData->OpenExisting( L".\\Data\\APeriodic\\Float",  L"float2" ) ;
  if( hr != S_OK ) return ;

  IIadsTime* sTime ;
  IIadsTime* eTime ;

  IadsData->get_SampleRate( &sampleRate );
  IadsData->get_StartTimeVB( &sTime );
  IadsData->get_EndTimeVB( &eTime);
  IadsData->get_DataFormat( &dataFormat) ;
  IadsData->get_DataType( &dataType ) ;
  IadsData->get_FileSize( &fileSize ) ;
  IadsData->get_TotalPoints( &totalPoints ) ;

  __int64 sNano, eNano ;
  sTime->get_TotalNanoseconds(&sNano) ;
  eTime->get_TotalNanoseconds(&eNano) ;

  printf( "Header Information:\n\n" );
  printf( "  Sample Rate: %f\n", sampleRate );
  printf( "    StartTime: %I64d\n", sNano );
  printf( "      EndTime: %I64d\n", eNano );
  printf( "  Data Format: %d\n", (int)dataFormat );
  printf( "    Data type: %d\n", (int)dataType );
  printf( "    File Size: %d\n", fileSize );
  printf( "Total Samples: %d\n", totalPoints );

  long actualPoints ;

    // Read and print out the first 50 data points from the file
    // 1) StartTime: -1.0 for a startime means to read fromthe start of the file leaving the pointer for the next read
    // 2) RequestedPonts - 50 total number of Points to read
    // 3) Data - the data Array
    // 4) Actual Points - The number of points that were read and returned in the data array
    // 5) TimeArray - Optional - filled in if not null - works for both periodic and Aperiodic

  VARIANT irigTime ; irigTime.vt = VT_EMPTY ;
  VARIANT data ; data.vt = VT_EMPTY ;

  hr = IadsData->ReadData( sTime, 600, &data, &actualPoints, &irigTime ) ;

  int p ;
  if( hr == S_OK )
  {
    IDispatch** pt = NULL ;
    IIadsTime* time = NULL ;
    __int64 t = -1 ;

    pt = (IDispatch**)irigTime.parray->pvData ;
    for( p=0; p < actualPoints; p++ )
    {
      if( pt && pt[p] )
      {
        pt[p]->QueryInterface(IID_IIadsTime, (void**)&time ) ;
        time->get_TotalNanoseconds(&t) ;
        time->Release() ;
      }

      float* d = (float*)data.parray->pvData ;
      printf( "%I64d, Data[%d], %f\n", t, p, d[p]  );
    }
  }
  IadsData->Close();

#endif
  
//
// Read the various periodic data types
//
#ifdef READ_PERIODIC_DATA

  printf("\n*) Read and Display Periodic Sample Files\n") ;

  // Read the first 50 values of each format type from the first parameter
  for( df=0; df < 5; df++ )
  {
    // Open existing IADS data file. Make sure the file is not "read only", because this funtion will fail.
    // It's written for general updating/creating of file only..
    if( df == 0 ) 
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Float",  bFileName[0] ) ;
    else if( df == 1 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Double", bFileName[0] ) ;
    else if( df == 2 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Int",    bFileName[0] ) ;
    else if( df == 3 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Int64",  bFileName[0] ) ;
    else if( df == 4 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Blob",  bFileName[0] ) ;
    else if( df == 5 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\UInt",  bFileName[0] ) ;
    else if( df == 6 )
      hr = IadsData->OpenExisting( L".\\Data\\Periodic\\UInt64",  bFileName[0] ) ;


    if( hr != S_OK ) continue;

    IadsData->get_SampleRate( &sampleRate );
    IadsData->get_StartTime( &startTime );
    IadsData->get_EndTime( &endTime);
    IadsData->get_DataFormat( &dataFormat) ;
    IadsData->get_DataType( &dataType ) ;
    IadsData->get_FileSize( &fileSize ) ;
    IadsData->get_TotalPoints( &totalPoints ) ;

    printf( "Header Information:\n\n" );
    printf( "Sample Rate: %f\n", sampleRate );
    printf( "StartTime: %I64d\n", startTime );
    printf( "EndTime: %I64d\n", endTime );
    printf( "Data Format: %d\n", (int)dataFormat );
    printf( "Data type: %d\n", (int)dataType );
    printf( "File Size: %d\n", fileSize );
    printf( "Total Points: %d\n", totalPoints );

    long actualPoints;

    // Read and print out the first 50 data points from the file
    // 1) StartTime: -1.0 for a startime means to read fromthe start of the file leaving the pointer for the next read
    // 2) RequestedPonts - 50 total number of Points to read
    // 3) Data - the data Array
    // 4) Actual Points - The number of points that were read and returned in the data array
    // 5) TimeArray - Optional - filled in if not null - works for both periodic and Aperiodic

    unsigned char b50Data[250] ;

    if( df == 0 ) 
      HRESULT hr = IadsData->ReadDataF32( startTime, 50, &fData[0], &actualPoints, NULL );
    else if( df == 1 )
      HRESULT hr = IadsData->ReadDataF64( startTime, 50, &dData[0], &actualPoints, NULL );
    else if( df == 2 )
      HRESULT hr = IadsData->ReadDataI32( startTime, 50, &iData[0], &actualPoints, NULL );
    else if( df == 3 )
      HRESULT hr = IadsData->ReadDataI64( startTime, 1, &i64Data[0], &actualPoints, NewTimeData );
    else if( df == 4 )
      HRESULT hr = IadsData->ReadDataBlob( startTime, 50, &b50Data[0], &actualPoints, NULL );
    else if( df == 5 )
      HRESULT hr = IadsData->ReadDataUI32( startTime, 50, &uiData[0], &actualPoints, NULL );
    else if( df == 6 )
      HRESULT hr = IadsData->ReadDataUI64( startTime, 50, &ui64Data[0], &actualPoints, NULL );

    if( hr == S_OK )
    {
      for( register int p=0; p < actualPoints; p++ )
      {
        if( df == 0 ) 
          printf( "Data[%d], %f\n", p, fData[p] );
        else if( df == 1 )
          printf( "Data[%d], %f\n", p, dData[p] );
        else if( df == 2 )
          printf( "Data[%d], %d\n", p, iData[p] );
        else if( df == 3 )
          printf( "%f, Data[%d], %I64d\n", NewTimeData[p], p, i64Data[p] );
        else if( df == 4 )
        {
          printf( "Data[%d], %02x-%02x-%02x-%02x-%02x\n", p, b50Data[p], b50Data[p+1], b50Data[p+2], b50Data[p+3], b50Data[p+4] );
          p+=5 ;
        }
        else if( df == 5 )
          printf( "Data[%d], %d\n", p, uiData[p] );
        else if( df == 6 )
          printf( "Data[%d], %d\n", p, ui64Data[p] );
      }
    }

    // Close the data file
    IadsData->Close();
  }

#endif

//
// Read the various periodic data types
//
#ifdef READ_PERIODIC_DATA2

  printf("\n*) Multi-Read Test and Display Periodic Sample Files\n") ;

  hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Float",  bFileName[0] ) ;
  if( hr != S_OK ) shutdown() ;

  IadsData->get_SampleRate( &sampleRate );
  IadsData->get_StartTime( &startTime );
  IadsData->get_EndTime( &endTime);
  IadsData->get_DataFormat( &dataFormat) ;
  IadsData->get_DataType( &dataType ) ;

  printf( "Header Information:\n\n" );
  printf( "Sample Rate: %f\n", sampleRate );
  printf( "StartTime: %I64d\n", startTime );
  printf( "EndTime: %I64d\n", endTime );
  printf( "Data Format: %d\n", (int)dataFormat );
  printf( "Data type: %d\n", (int)dataType );


  for( register int n=0; n < 5000; n++ ) NewTimeData[n] = -1 ;

  long actualPoints;
  hr = IadsData->ReadDataF32( startTime, 10, fData, &actualPoints, NewTimeData );
  if( hr == S_OK )
  {
    for( register int p=0; p < actualPoints; p++ )
    {
      printf( "%d. Time:%I64d, Data:%f\n", p, NewTimeData[p], fData[p] );
    }

  }

  hr = IadsData->ReadDataF32( NewTimeData[9], 10, fData, &actualPoints, NewTimeData );
  if( hr == S_OK )
  {
    for( register int p=0; p < actualPoints; p++ )
    {
      printf( "%d. Time:%I64d, Data:%f\n", p, NewTimeData[p], fData[p] );
    }

  }

  hr = IadsData->ReadDataF32( startTime, 10, fData, &actualPoints, NewTimeData );
  if( hr == S_OK )
  {
    for( register int p=0; p < actualPoints; p++ )
    {
      printf( "%d. Time:%I64d, Data:%f\n", p, NewTimeData[p], fData[p] );
    }

  }

  // Try a GetNext call to make sure it's starting where the last read ended
  hr = IadsData->ReadDataF32( -1, 10, fData, &actualPoints, NewTimeData );
  if( hr == S_OK )
  {
    for( register int p=0; p < actualPoints; p++ )
    {
      printf( "%d. Time:%I64d, Data:%f\n", p, NewTimeData[p], fData[p] );
    }

  }



  // Close the data file
  IadsData->Close();

#endif



//
// Read Periodic data using the "Get Next" style read - where StartTime is set to -1.0
//
#ifdef READ_GET_NEXT_STYLE

  printf("\n*) Read and Display Using Get Next Strategy\n") ;

  //
  // Show how to read through the data from start to end using the startTime set to -1.0 
  //
  hr = IadsData->OpenExisting( L".\\Data\\Periodic\\Float", bFileName[0] );

  // Read the properties
  IadsData->get_SampleRate( &sampleRate );
  IadsData->get_StartTime( &startTime );
  IadsData->get_EndTime( &endTime);
  IadsData->get_DataFormat( &dataFormat) ;
  IadsData->get_DataType( &dataType) ;

  printf( "Header Information:\n\n" );
  printf( "Sample Rate: %f\n", sampleRate );
  printf( "  StartTime: %I64d\n", startTime );
  printf( "    EndTime: %I64d\n", endTime );
  printf( "Data Format: %d\n", (int)dataFormat );
  printf( "  Data Type: %d\n", (int)dataType );

  int cnt = 0 ;
  while( 1 )
  {
    long actualPoints ;
    HRESULT hr = IadsData->ReadDataF32( -1, 50, fData, &actualPoints, timeData );
    if( hr != S_OK || actualPoints < 1 ) break ;
    for( int n=0; n < 5; n++ )
      printf( "Time:%I64d, Data:%f\n", timeData[n], fData[n] );
  }

  printf("Rewind and show again...\n") ;

  // Check the rewind function
  IadsData->Rewind() ;

  //FILE* fp = fopen("c:\\data.txt", "w") ;
  while( 1 )
  {
    long actualPoints ;
    HRESULT hr = IadsData->ReadDataF32( -1, 50, fData, &actualPoints, timeData );
    if( hr != S_OK || actualPoints < 1 )
      break ;
    for( int n=0; n < actualPoints; n++ )
    {
      printf( "Time:%I64d, Data:%f\n", timeData[n], fData[n] );
      //if( fp )
        //fprintf( fp, "Time:%f, Time: %I64d, Data:%f\n", timeData[n], (_int64)(timeData[n]*cNanoSec), fData[n] );
    }
  }

  //fclose(fp) ;
  IadsData->Close() ;

#endif

//
// Read APeriodic data
//
#ifdef READ_APERIODIC_DATA

  printf("\n*) Read and Display Aperiodic Data using Get Next strategy\n") ;

  //
  // Show how to read through the data from start to end using the startTime set to -1.0 
  //
  hr = IadsData->OpenExisting( L".\\Data\\APeriodic\\Float", bFileName[0] );

  // Read the properties
  IadsData->get_SampleRate( &sampleRate );
  IadsData->get_StartTime( &startTime );
  IadsData->get_EndTime( &endTime);
  IadsData->get_DataFormat( &dataFormat) ;
  IadsData->get_DataType( &dataType) ;

  printf( "Header Information\n\n" );
  printf( "Sample Rate: %f\n", sampleRate );
  printf( "  StartTime: %I64d\n", startTime );
  printf( "    EndTime: %I64d\n", endTime );
  printf( "Data Format: %d\n", (int)dataFormat );
  printf( "Data Type: %d\n", (int)dataType );

  // Use get next technique
  long actualPoints ;

  __int64 freshTimeData[5000] ;
  float  freshFloatData[5000] ;

  while( 1 )
  {
    HRESULT hr = IadsData->ReadDataF32( -1, 5000, freshFloatData, &actualPoints, freshTimeData );
    if( hr != S_OK || actualPoints < 1 )
      break ;
    for( int n=0; n < 50; n++ )
      printf( "%I64d, Data: %f\n", freshTimeData[n], freshFloatData[n] );
  }

  printf("\n ** Goto Time **\n\n") ;

  // Use goto time technique
  hr = IadsData->ReadDataF32( 86450*cNanoSec, 50, fData, &actualPoints, timeData );
  if( hr == S_OK && actualPoints > 0 )
  {
    for( register int n=0; n < actualPoints; n++ )
      printf( "%I64d, Data: %f\n", timeData[n], fData[n] );
  }

  IadsData->Close() ;

#endif


//
// Multi-Periodic files
//
#ifdef READ_MULTI_PERIODIC_DATA

  printf("\n*) Read and Display Multi-Periodic Sample Files\n") ;

  // Read the first 50 values of each format type from the first parameter
  for( df=0; df < 2; df++ )
  {
    // Open existing IADS data file. Make sure the file is not "read only", because this funtion will fail.
    // It's written for general updating/creating of file only..
    if( df == 0 ) 
      hr = IadsData->OpenExisting( L".\\Data\\MultiPeriodic\\Float",  bFileName[0] ) ;
    else if( df == 1 )
      hr = IadsData->OpenExisting( L".\\Data\\MultiPeriodic\\Double", bFileName[0] ) ;

    if( hr != S_OK ) continue;

    IadsData->get_SampleRate( &sampleRate );
    IadsData->get_StartTime( &startTime );
    IadsData->get_EndTime( &endTime);
    IadsData->get_DataFormat( &dataFormat) ;
    IadsData->get_DataType( &dataType) ;
    
    printf( "Header Information:\n\n" );
    printf( "Sample Rate: %f\n", sampleRate );
    printf( "StartTime: %I64d\n", startTime );
    printf( "EndTime: %I64d\n", endTime );
    printf( "Data Format: %d\n", (int)dataFormat );
    printf( "Data Type: %d\n", (int)dataType );

    long actualPoints;

    if( df == 0 )
    {
      HRESULT hr = IadsData->ReadDataF32( 86999*cNanoSec, 200, fData, &actualPoints, timeData );
      if( hr == S_OK ) 
      {
        for( register int p=0; p < actualPoints; p++ )
          printf( "%I64d, Data[%d], %f\n", timeData[p], p, fData[p] );
      }
    }
    else
    {
      HRESULT hr = IadsData->ReadDataF64( 86499*cNanoSec, 200, dData, &actualPoints, timeData );
      if( hr == S_OK )
      {
        for( register int p=0; p < actualPoints; p++ )
          printf( "%I64d, Data[%d], %f\n", timeData[p], p, fData[p] );
      }
    }

    // Close the data file
    IadsData->Close();
  }

#endif


//
// Demonstrate reading our legcay file type with this same API
//
#ifdef READ_CLASSIC_DATA

  printf("\n*) Read Classic and Display Classic IADS data using Get Next Strategy\n") ;

  //
  // Show how to read through the data from start to end using the startTime set to -1.0 
  //
  hr = IadsData->OpenExisting( L".\\Data\\Classic\\", L"Parm1.LittleEndianCDSData" );
  if( hr == S_OK ) 
  {
    // Read the properties
    IadsData->get_SampleRate( &sampleRate );
    IadsData->get_StartTime( &startTime );
    IadsData->get_EndTime( &endTime);

    printf( "Header Information:\n\n" );
    printf( "Sample Rate: %f\n", sampleRate );
    printf( "  StartTime: %I64d\n", startTime );
    printf( "    EndTime: %I64d\n", endTime );

    int cnt = 0 ;
    while( 1 )
    {
      long actualPoints ;
      HRESULT hr = IadsData->ReadDataF32( -1, 50, &fData[0], &actualPoints, NULL );
      if( hr != S_OK || actualPoints < 1 )
        break ;
      printf( "Data(at index %d): %f\n", cnt++*50, fData[0] );
    }
    IadsData->Close() ;
  }
#endif

//
// Check the script Interface calls from C++ - what the heck
//
#ifdef SCRIPT_METHODS_TEST

  printf("\n*) Create Periodic data using Script Style interface method - AppendDataVariant \n") ;

  // Data
  SAFEARRAY *psa = ::SafeArrayCreateVector(4, 0, 5000 ) ;
  if( !psa ) shutdown() ;
  for( int i=0; i < 5000; i++ ) fData[i] = (float)i ;
  psa->pvData = fData ;
  VARIANT vData ; vData.parray = psa ; vData.vt = VT_ARRAY|VT_R4 ;
 
  // Time data
  SAFEARRAY *psa2 = ::SafeArrayCreateVector(5, 0, 5000 ) ;
  if( !psa2 ) shutdown() ;

  timeData[0] = 86400*cNanoSec ;
  minTimeInSec = (__int64)((1.0/200.0)*1E9) ;
  slope = 200.0/(float)RAND_MAX ;
  for(int t2=1; t2 < 5000; t2++ )
  {
    double y = slope * (float)::rand() ;
    timeData[t2] = timeData[t2-1] + (__int64)(y*minTimeInSec) ; 
  }
  psa2->pvData = timeData ;
  VARIANT vTime ; vTime.parray = psa2 ; vTime.vt = VT_ARRAY|VT_R8 ;

  // Periodic using the Variant interface
  hr = IadsData->Create( L".\\Data\\ScriptStyle",  L"parm1" ) ;
  if( hr == S_OK )
  {
    IadsData->put_StartTime( 86400*cNanoSec ); // we want all files to start at the same time
    IadsData->put_SampleRate( 50.0 ) ;
    IadsData->put_DataType( IadsPeriodicType ) ;
    IadsData->put_DataFormat( IadsFloat32Type ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      IadsData->AppendDataVariant( 5000, &vData, &anp ) ;
    }
    IadsData->Close() ;
  }

  // Periodic using the Variant2 interface
  hr = IadsData->Create( L".\\Data\\ScriptStyle",  L"parm2" ) ;
  if( hr == S_OK )
  {
    IadsData->put_StartTime( 86400*cNanoSec ); // we want all files to start at the same time
    IadsData->put_SampleRate( 50.0 ) ;
    IadsData->put_DataType( IadsPeriodicType ) ;
    IadsData->put_DataFormat( IadsFloat32Type ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      IadsData->AppendDataVariant( 5000, &vData, &anp ) ;
    }
    IadsData->Close() ;
  }

  // Aperiodic data using the Variant2 interface
  hr = IadsData->Create( L".\\Data\\ScriptStyle",  L"parm3" ) ;
  if( hr == S_OK )
  {
    IadsData->put_StartTime( timeData[0] );
    IadsData->put_SampleRate( 0.0 ) ;
    IadsData->put_DataType( IadsAsyncType ) ;
    IadsData->put_DataFormat( IadsFloat32Type ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      hr = IadsData->AppendDataVariant( 5000, &vData, &anp ) ;
    }
    IadsData->Close() ;
  }

  ::SafeArrayDestroyData( psa ) ;
  ::SafeArrayDestroyData( psa2 ) ;

#endif

//
//  Check start time processing
//
#ifdef START_TIME_TEST

  printf("\n*) Start Time Test\n") ;

  timeData[0] = (__int64)(19138415.045183998*1E9)  ;

  minTimeInSec = 1.0/200.0 ;
  slope = 200.0/(float)RAND_MAX ;
  for(int t=1; t < 5000; t++ )
  {
    double y = slope * (float)::rand() ;
    timeData[t] = timeData[t-1] + (__int64)((y*minTimeInSec)*1E9) ; 
  }

  if( IadsData->Create( L".\\Data\\StartTimeTest",  L"Parm1" ) == S_OK )
  {
    IadsData->put_StartTime( timeData[0] ); // we want all files to start at the same time
    // Although the samlpe rate is not required - this allows the user to set the 
    // suggested iteration sample rate for displays that will continue to fill data, like strip charts.
    IadsData->put_SampleRate( 50.0 ) ; 
    IadsData->put_DataFormat( IadsFloat32Type ) ;
    IadsData->put_DataType( IadsAsyncType ) ;

    if( IadsData->WriteHeader() == S_OK )
    {
      // This automatically updates the data file and index file
      IadsData->AppendDataF32( 5000, fData, NULL, timeData ) ;
    }
    IadsData->Close() ;
  }


#endif

  shutdown() ;
}

